/// <reference types="node" />

import { Container, Context, IPriorityEventEmitter, Identity, Kernel, Logger, ObjectId, Resolvable, Route, Sequence, Settings, StepFunction } from '@agentframework/app';
import { Constructor, IAttribute, Property } from '@agentframework/core';
import { IBlobLike, IProfile, Model, PrimaryKey } from '@agentframework/data';
import { ClassConstructor, ClassInitializer } from '@agentframework/domain';
import { ControllerAttribute, MiddlewareAttribute, PerformanceTimer, WebApplication, WebApplicationSettings, WebContext, WebRequest, WebRequestIdentity, WebRouter, WebRouterHandler } from '@agentframework/web';

export interface EmailProvider {
	provider: string;
	domains: Array<string>;
}
export declare class WebFoundationSettings extends WebApplicationSettings {
	API_ROOT: string;
	SECRET_KEYS: Array<string>;
	WEB_DEVICE_COOKIE_DOMAIN: string;
	WEB_DEVICE_COOKIE_KEY: string;
	WEB_DEVICE_COOKIE_EXPIRES_IN_SECONDS: number;
	WEB_SESSION_COOKIE_KEY: string;
	WEB_SESSION_COOKIE_EXPIRES_IN_SECONDS: number;
	WEB_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEB_CONTENT_SECURITY_POLICY: Array<string>;
	WEB_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEB_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_RESEND_COOLDOWN_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_SIGNIN_PATH: string;
	WEB_SIGNIN_SUCCESS_PATH: string;
	WEB_SIGNIN_FAILED_PATH: string;
	WEB_ACCESS_DENIED_PATH: string;
	WEB_SIGNOUT_PATH: string;
	WEB_SIGNOUT_COMPLETE_PATH: string;
	ADMIN_EMAIL: string;
	TRUSTED_EMAIL_PROVIDERS: Array<EmailProvider>;
	THEME: string;
	OWNER_NAME: string;
	OWNER_COMPANY_NAME: string;
	OWNER_COMPANY_CODE: string;
	OWNER_COMPANY_ADDRESS: Array<string>;
	OWNER_COMPANY_TELEPHONE: string;
	OWNER_COMPANY_FAX: string;
	OWNER_AGREEMENTS: Array<any>;
	OIDC_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OIDC_REDIRECT_LIST: Array<string>;
	OIDC_NO_REDIRECTS_LIST: Array<string>;
	JSONSDK_FETCH_API_URL: string;
	JSONSDK_FETCH_API_KEY: string;
	JSONSDK_FETCH_API_SECRET: string;
	OIDC_MICROSOFT: boolean;
	OIDC_MICROSOFT_ISSUER_URL: string;
	OIDC_MICROSOFT_CLIENT_ID: string;
	OIDC_MICROSOFT_CLIENT_SECRET: string;
	OIDC_MICROSOFT_SCOPES: Array<string>;
	OIDC_BITBUCKET: boolean;
	OIDC_BITBUCKET_ISSUER_URL: string;
	OIDC_BITBUCKET_CLIENT_ID: string;
	OIDC_BITBUCKET_CLIENT_SECRET: string;
	OIDC_BITBUCKET_SCOPES: Array<string>;
	OIDC_GOOGLE: boolean;
	OIDC_GOOGLE_ISSUER_URL: string;
	OIDC_GOOGLE_CLIENT_ID: string;
	OIDC_GOOGLE_CLIENT_SECRET: string;
	OIDC_GOOGLE_SCOPES: Array<string>;
	OIDC_GITLAB: boolean;
	OIDC_GITLAB_ISSUER_URL: string;
	OIDC_GITLAB_CLIENT_ID: string;
	OIDC_GITLAB_CLIENT_SECRET: string;
	OIDC_GITLAB_SCOPES: Array<string>;
	OIDC_GITHUB: boolean;
	OIDC_GITHUB_ISSUER_URL: string;
	OIDC_GITHUB_CLIENT_ID: string;
	OIDC_GITHUB_CLIENT_SECRET: string;
	OIDC_GITHUB_SCOPES: Array<string>;
	OIDC_WEIBO: boolean;
	OIDC_WEIBO_ISSUER_URL: string;
	OIDC_WEIBO_CLIENT_ID: string;
	OIDC_WEIBO_CLIENT_SECRET: string;
	OIDC_WEIBO_SCOPES: Array<string>;
	CUSTOM_WECHAT_APP_NAME: string;
	CUSTOM_WECHAT_APP_ID: string;
	CUSTOM_WECHAT_APP_SECRET: string;
	CUSTOM_GOOGLE_API_KEY: string;
	CUSTOM_GOOGLE_ANALYTICS_ID: string;
	CUSTOM_GOOGLE_API_EMAIL_ID: string;
	CUSTOM_GOOGLE_API_PRIVATE_KEY: string;
	CUSTOM_AWS_ACCESS_KEY_ID: string;
	CUSTOM_AWS_SECRET_ACCESS_KEY: string;
	CUSTOM_AWS_REGION: string;
	CUSTOM_STRIPE_PUBLIC_KEY: string;
	CUSTOM_STRIPE_SECRET_KEY: string;
	CUSTOM_MAILGUN_API_KEY: string;
	CUSTOM_KUBERNETES_API_URL: string;
	CUSTOM_KUBERNETES_API_TOKEN: string;
	TELEGRAM_BOT_ID: string;
	TELEGRAM_BOT_NAME: string;
	TELEGRAM_BOT_DISPLAY_NAME: string;
	TELEGRAM_BOT_TOKEN: string;
	TELEGRAM_BOT_SECRET_KEY: string;
	TELEGRAM_BOT_DIALOG_TIMEOUT_IN_SECONDS: number;
}
export interface WebReq extends WebRequest<WebFoundationSettings> {
}
export declare class WebFoundationRequestIdentity extends WebRequestIdentity {
	static [ClassInitializer](req: WebReq, params: Array<any>): WebFoundationRequestIdentity;
	session?: string;
	user?: string;
	token?: string;
	roles?: Array<string>;
	groups?: Array<string>;
	scope?: Array<string>;
	profile?: IProfile;
	timezone: string;
	locale: string;
	theme: string;
}
export interface WebFoundationRequest<S extends WebFoundationSettings> extends WebRequest<S> {
	identity: WebFoundationRequestIdentity;
}
export interface WebFoundationContext<S extends WebFoundationSettings> extends WebContext<S> {
	identity: WebFoundationRequestIdentity;
	req: WebFoundationRequest<S>;
}
export interface WebCtx extends WebFoundationContext<WebFoundationSettings> {
}
export interface IAuthenticationToken {
	token_type: string;
	scope: string;
	expires_in: number;
	ext_expires_in: number;
	access_token: string;
	refresh_token: string;
	id_token: any | null;
}
export declare type ProviderProtocols = string | 'openid' | 'oauth' | 'oauth2' | 'bot';
export interface IExternalProviderPrimaryKey {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	appID: string;
}
export interface IExternalProviderProfile {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID?: string;
	appSecret?: string;
	scope?: string;
	displayName?: string;
	displayImageURL?: string;
	description?: string;
}
export interface IExternalProvider extends IExternalProviderPrimaryKey, IProfile {
	_id: ObjectId;
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
}
declare class ExternalProviderModel extends Model implements IExternalProvider {
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerFingerprint: string;
	providerConfiguration: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
	static toJSON(a: any): any;
	fromJSON(json: any): void;
}
export interface IExternalUserIdentity {
	type: string;
	key?: string;
	value: string;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	providerId: string;
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: any;
	userData: any;
}
export interface IExternalUserOptionalFields {
	user?: string;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface IExternalUser extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	_id: ObjectId;
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user?: string;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IProfile;
	updatedAt?: Date;
	updatedFrom?: string;
	updatedBy?: IProfile;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IProfile;
	deletedReason?: string;
}
export interface IAuthenticationProfile extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	uid: number | string;
	provider: string;
	providerProtocol: string;
	providerId: string;
	providerData: any;
	userData: any;
	identities: Array<IExternalUserIdentity>;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
declare class WebFoundationController extends Resolvable<WebFoundationSettings> {
	protected getFailureURL(ctx: WebCtx, err: Error): string;
}
export declare class WebFoundationService extends Resolvable<WebFoundationSettings> {
}
declare class ExternalUserModel extends Model implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: ProviderProtocols;
	providerURL: string;
	providerId: string;
	providerData: string;
	userData: string;
	user: any;
	username: string;
	domain: string;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string;
	displayImageFile: string;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface Ctx extends Context<WebFoundationSettings> {
}
declare class UserModel extends Model {
	id: number;
	name: string;
	displayName: string;
	displayImageURL: string;
	displayImageFile: string;
	signature?: any;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: any;
	theme?: string;
	timezone?: string;
	locale?: string;
	password: string;
	passwordSalt: string;
	paymentPassword: string;
	paymentPasswordSalt: string;
}
export declare class ExternalUserService extends WebFoundationService {
	connectUser(ctx: Ctx, externalUser: ExternalUserModel, user: UserModel): Promise<boolean>;
	findOrCreateExternalUser(ctx: Ctx, profile: IAuthenticationProfile): Promise<ExternalUserModel>;
	findOrCreateOrUpdateDifferenceExternalUser(ctx: Ctx, profile: IAuthenticationProfile): Promise<ExternalUserModel>;
}
declare class UserService extends WebFoundationService {
	findOneById(ctx: Ctx, id: PrimaryKey): Promise<UserModel | undefined>;
	isEmailBeenUsed(ctx: Ctx, email: string): Promise<boolean>;
	isMobileExists(ctx: Ctx, mobile: string): Promise<boolean>;
	findUserFromEmail(ctx: Ctx, email: string): Promise<UserModel | undefined>;
	findOrCreateUserFromExternalUser(ctx: Ctx, externalUser: IExternalUser): Promise<UserModel>;
}
declare abstract class OpenIdController extends WebFoundationController {
	protected ExternalUsers: ExternalUserService;
	protected Users: UserService;
	protected abstract discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	private getAuthenticator;
	protected OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<IAuthenticationProfile | null>;
	protected abstract OnAuthenticated(ctx: WebCtx, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUsers?: UserModel, loggedInUser?: UserModel): Promise<UserModel>;
}
declare class IdentityProfile {
	_id: ObjectId;
	id: number;
	displayName: string;
	displayImageURL: string;
}
export interface ISession {
	session: string;
	user: string;
	token: string;
	roles: Array<string>;
	groups?: Array<string>;
	scope?: Array<string>;
	profile: IdentityProfile;
	timezone?: string;
	locale?: string;
	theme?: string;
}
export declare class SessionService extends WebFoundationService {
	CreateSession(req: WebReq, session: Partial<ISession>): Promise<ISession>;
	ValidateSession(req: WebReq): Promise<boolean>;
	UpdateSession(req: WebReq, id: string, session: Partial<ISession>): Promise<boolean>;
	ExtendSession(req: WebReq, id: string): Promise<boolean>;
	TerminateSession(req: WebReq, id: string, reason: string): Promise<boolean>;
	private static CreateNewSessionId;
	private static CreateNewTokenId;
}
declare class AuthenticationService extends WebFoundationService {
	protected Sessions: SessionService;
	IsAuthenticated(req: WebReq): Promise<boolean>;
	SignIn(req: WebReq, user: UserModel, allows?: Array<string>): Promise<ISession | undefined>;
	Update(req: WebReq, user: UserModel): Promise<boolean>;
	Refresh(req: WebReq): Promise<boolean>;
	SignOut(req: WebReq, reason?: string): Promise<boolean>;
}
declare enum EventType {
	SIGN_UP = "sign-up",
	SIGN_UP_FAILED = "sign-up-failed",
	SIGN_IN = "sign-in",
	SIGN_IN_FAILED = "sign-in-failed",
	SIGN_OUT = "sign-out",
	SESSION_EXPIRED = "session-expired",
	ACCESS_DENIED = "access-denied"
}
declare class EventModel extends Model {
	type: EventType;
	context: any;
}
declare class EventService extends WebFoundationService {
	getLatestEvents(ctx: Ctx): Promise<Array<EventModel>>;
	createSignInEvent(ctx: Ctx, idProvider: ExternalProviderModel, user: UserModel): Promise<EventModel | undefined>;
	createSignUpEvent(ctx: Ctx, idProvider: ExternalProviderModel, user: UserModel): Promise<EventModel | undefined>;
	createSignOutEvent(ctx: Ctx, idProvider: ExternalProviderModel): Promise<EventModel | undefined>;
}
declare abstract class OpenIdAuthController extends OpenIdController {
	protected Events: EventService;
	protected WebAuth: AuthenticationService;
	OnAuthenticated(ctx: WebCtx, externalProvider: ExternalProviderModel, token: IAuthenticationToken, externalUser: ExternalUserModel, foundUser?: UserModel, loggedInUser?: UserModel): Promise<UserModel>;
}
export declare class ExternalProviderService extends WebFoundationService {
	findOrResolveExternalProvider(ctx: Ctx, profile: any): Promise<ExternalProviderModel>;
	protected createProviderId(profile: IExternalProviderProfile): ObjectId;
	protected discoverProviderConfiguration(ctx: Ctx, providerURL: string): Promise<any>;
}
declare class WeiboProfileService {
	getProfile(domain: Ctx, token: any, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class WeiboOpenIdAuthController extends OpenIdAuthController {
	idp: ExternalProviderService;
	weibo: WeiboProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
declare class BitBucketProfileService {
	getProfile(ctx: Ctx, token: any, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class BitBucketOpenIdAuthController extends OpenIdAuthController {
	protected idp: ExternalProviderService;
	protected profile: BitBucketProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export interface GoogleIdToken {
	azp: string;
	aud: string;
	sub: string;
	email: string;
	email_verified: boolean;
	at_hash: string;
	exp: number;
	iss: string;
	iat: number;
	name: string;
	picture: string;
	given_name: string;
	family_name: string;
	locale: string;
}
export interface GoogleToken extends IAuthenticationToken {
	id_token: GoogleIdToken;
}
declare class GoogleProfileService {
	TrustedEmailDomains: string[];
	getProfile(ctx: Ctx, token: GoogleToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GoogleOpenIdAuthController extends OpenIdAuthController {
	protected idp: ExternalProviderService;
	protected google: GoogleProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: ExternalProviderModel): Promise<IAuthenticationProfile | null>;
}
declare class GitLabProfileService {
	getProfile(req: Ctx, token: any, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GitLabOpenIdAuthController extends OpenIdAuthController {
	protected idp: ExternalProviderService;
	protected gitlab: GitLabProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
declare class GitHubProfileService {
	getProfile(req: Ctx, token: any, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class GitHubOpenIdAuthController extends OpenIdAuthController {
	protected idp: ExternalProviderService;
	protected github: GitHubProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export interface MicrosoftIdToken {
	ver: string;
	iss: string;
	sub: string;
	aud: string;
	exp: number;
	iat: number;
	nbf: number;
	name: string;
	preferred_username: string;
	oid: string;
	email: string;
	tid: string;
	aio: string;
}
export interface MicrosoftToken extends IAuthenticationToken {
	id_token: MicrosoftIdToken;
}
declare class MicrosoftProfileService {
	TrustedMicrosoftEmailDomains: string[];
	getProfile(ctx: Ctx, token: MicrosoftToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare class MicrosoftOpenIdAuthController extends OpenIdAuthController {
	protected idp: ExternalProviderService;
	protected profile: MicrosoftProfileService;
	discover(ctx: WebCtx): Promise<ExternalProviderModel>;
	OnProfile(ctx: WebCtx, token: IAuthenticationToken, provider: IExternalProvider): Promise<IAuthenticationProfile | null>;
}
export declare abstract class WebFoundationApplication<S extends WebFoundationSettings> extends WebApplication<S> {
	protected onAfterInit(): void;
}
export declare function dialog(name?: string): MethodDecorator;
export declare class DialogAttribute implements IAttribute {
	private _name;
	constructor(_name: string);
	beforeDecorate(target: Object, propertyKey: string | symbol, descriptor: PropertyDescriptor): boolean;
	readonly name: string;
}
export declare class TelegramBotController extends WebFoundationController {
	private token;
	private bots;
	setup(ctx: WebCtx): Promise<any>;
	telegram(ctx: WebCtx): Promise<void>;
}
declare class State {
}
export interface BotState {
	userState: State;
	conversationState: State;
	privateConversationState: State;
	dialogState: State;
}
declare class BotStateService extends Resolvable<Settings> {
	getUserState(user: string): Promise<State>;
	getConversationState(conversation: string): Promise<State>;
	getPrivateConversationState(user: string, conversation: string): Promise<State>;
	getDialogState(user: string, conversation: string, dialog: string): Promise<State>;
	getState(user: string, conversation: string, dialog: string): Promise<BotState>;
	updateState(state: BotState): Promise<any>;
}
declare class TypeRouterBindings extends Sequence {
	instance: any;
	type: Constructor<any>;
	methods: Array<any>;
}
export interface DialogRouterEventHandler {
	beforeMiddleware?(controller: ControllerAttribute, middleware: MiddlewareAttribute, property: Property): boolean;
	beforeDialog?(controller: ControllerAttribute, dialog: DialogAttribute, property: Property): boolean;
}
declare class BotDialogRouter extends WebRouter {
	protected _dialogs: Array<any>;
	protected _root: Route;
	search(path: string): WebRouterHandler | undefined;
	add(dialog: Constructor<any>, handlers?: DialogRouterEventHandler): this;
	protected createMiddleware(type: Constructor<any>): TypeRouterBindings;
	protected addMiddleware(bindings: TypeRouterBindings, property: Property, idx: number): void;
	protected addDialog(bindings: TypeRouterBindings, property: Property, node: Route, idx: number): void;
	private makeInvokeMethod;
	private generateInitializerFn;
	private generateMiddlewareFn;
	private generateDialogFn;
}
declare class BotServiceEventHandler extends Resolvable<Settings> implements DialogRouterEventHandler {
	beforeMiddleware(controller: ControllerAttribute, middleware: MiddlewareAttribute, property: Property): boolean;
	beforeDialog(controller: ControllerAttribute, dialog: DialogAttribute, property: Property): boolean;
	private makeMiddlewareSignature;
	private makeMethodSignature;
}
export interface BotEventUser {
	id: string | number;
	username: string;
	displayName: string;
	firstName?: string;
	lastName?: string;
	bot?: boolean;
	locale?: string;
	timezone?: string;
}
declare abstract class BotEvent {
	abstract readonly id: string;
	abstract readonly type: string;
	abstract readonly timestamp: Date;
	abstract readonly from: BotEventUser;
	abstract readonly conversation: {
		id: string;
		isGroup: boolean;
	};
	abstract readonly recipient: BotEventUser;
	abstract readonly channelId: string;
	abstract readonly channelData: any;
	abstract readonly text?: string;
	abstract readonly data?: any;
	abstract readonly dataType?: string;
}
export interface MessageProfile extends IProfile {
	_id: string;
	id: string;
	bot?: boolean;
}
declare class MessageIdentity extends Identity {
	id: string;
	user: string;
	profile: MessageProfile;
}
export interface BotSettings extends WebFoundationSettings {
	TELEGRAM_BOT_ID: string;
	TELEGRAM_BOT_NAME: string;
	TELEGRAM_BOT_DISPLAY_NAME: string;
	TELEGRAM_BOT_TOKEN: string;
	TELEGRAM_BOT_SECRET_KEY: string;
	TELEGRAM_BOT_DIALOG_TIMEOUT_IN_SECONDS: number;
}
declare abstract class BotMessage {
	abstract readonly text?: string;
	abstract readonly command?: string;
	abstract readonly commandArgs: Array<string>;
}
declare class DialogAction {
}
declare class DialogRouteAction extends DialogAction {
	constructor();
}
declare class DialogResponseAction extends DialogAction {
	constructor();
}
export declare abstract class BotContext<S extends BotSettings> extends Container<S> {
	constructor();
	readonly performance: PerformanceTimer;
	readonly state: any;
	readonly emitter: IPriorityEventEmitter;
	readonly logger: Logger;
	readonly settings: S;
	abstract readonly identity: MessageIdentity;
	abstract readonly event: BotEvent;
	path: string;
	pathIndex: number;
	pathIndexMax: number;
	data: any;
	readonly type: string;
	abstract readonly message: BotMessage;
	sendText(text: string, user?: string): Promise<any>;
	startDialog(dialogPath: string, data?: any): DialogRouteAction;
	replaceDialog(dialogPath: string, data?: any): DialogRouteAction;
	endDialog(data?: any): DialogRouteAction;
	endConversation(): DialogRouteAction;
}
export declare class BotService<S extends BotSettings> extends Resolvable<S> {
	protected readonly states: BotStateService;
	protected readonly router: BotDialogRouter;
	protected readonly handler: BotServiceEventHandler;
	protected readonly middleware: Sequence;
	protected onInit(): void;
	invoke(btx: BotContext<S>): Promise<DialogResponseAction | any>;
	addMiddleware(step: StepFunction): void;
	addDialog(dialog: ClassConstructor<any>): void;
	addDialogs(namespace: any): void;
}
export declare class TelegramBotService extends BotService<BotSettings> {
	protected onInit(): void;
}
declare class TelegramBotEvent extends BotEvent {
	readonly channelData: any;
	readonly channelId: string;
	readonly conversation: {
		id: string;
		isGroup: boolean;
	};
	readonly from: BotEventUser;
	readonly id: string;
	readonly recipient: BotEventUser;
	readonly timestamp: Date;
	readonly type: string;
	readonly text: string | undefined;
	readonly data: any | undefined;
	readonly dataType: string | undefined;
	static [ClassInitializer](req: WebRequest<WebApplicationSettings>, [recipient]: Array<any>): Promise<{
		id: any;
		type: string;
		timestamp: Date;
		from: BotEventUser;
		conversation: {
			isGroup: boolean;
			id: any;
		};
		recipient: any;
		channelId: string;
		channelData: any;
		text: string | undefined;
		data: any;
		dataType: string;
	}>;
}
declare class TelegramMessageIdentity extends MessageIdentity {
	static [ClassInitializer](req: WebReq, parameters: Array<any>): Promise<TelegramMessageIdentity>;
}
declare class TelegramBotMessage extends BotMessage {
	private event;
	constructor(event: TelegramBotEvent);
	readonly text: string | undefined;
	readonly command: string | undefined;
	readonly commandArgs: Array<string>;
}
export interface InitFetchOptions {
	baseURL?: string;
	headers?: any;
	query?: any;
}
export interface FetchOptions {
	method?: string;
	url?: string;
	headers?: any;
	query?: any;
	data?: any;
	raw?: boolean;
}
export interface Headers extends Iterable<[string, string]> {
	forEach(callback: (value: string, name: string) => void): void;
	append(name: string, value: string): void;
	delete(name: string): void;
	get(name: string): string | null;
	getAll(name: string): string[];
	has(name: string): boolean;
	raw(): {
		[k: string]: string[];
	};
	set(name: string, value: string): void;
	entries(): Iterator<[string, string]>;
	keys(): Iterator<string>;
	values(): Iterator<[string]>;
	[Symbol.iterator](): Iterator<[string, string]>;
}
export interface Body {
	arrayBuffer(): Promise<ArrayBufferLike>;
	body: NodeJS.ReadableStream;
	bodyUsed: boolean;
	buffer(): Promise<Buffer>;
	json(): Promise<any>;
	text(): Promise<string>;
}
export interface FetchResponse {
	url: string;
	status: number;
	statusText: string;
	type: string;
	headers: Headers;
	body: Body;
	data?: any;
}
export declare class ExternalFetcher {
	protected readonly defaults: InitFetchOptions;
	constructor(defaults: InitFetchOptions);
	private makeFetchOptions;
	protected httpRequest(options: FetchOptions): Promise<FetchResponse>;
	protected httpPost(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	protected httpPut(url: string, data: any, options?: FetchOptions): Promise<FetchResponse>;
	protected httpGet(url: string, options?: FetchOptions): Promise<FetchResponse>;
	protected httpGetAsText(url: string, options?: FetchOptions): Promise<string>;
	protected httpGetAsBuffer(url: string, options?: FetchOptions): Promise<Buffer>;
	protected httpGetAsJson(url: string, options?: FetchOptions): Promise<any>;
	protected httpDelete(url: string, options?: FetchOptions): Promise<FetchResponse>;
}
export interface getWebhookInfoResult {
	url: string;
	has_custom_certificate: boolean;
	pending_update_count: number;
	max_connections: number;
}
export interface ResponseFromServer<T> {
	ok: 1 | 0;
	result: T;
	data: {
		ok: boolean;
		result: T;
	};
}
export interface InMemoryDocument {
	filename?: string;
	type: string;
	data: Buffer;
}
declare class TelegramClient extends ExternalFetcher {
	constructor(apiKey: any);
	static [ClassInitializer](owner: Kernel, params: Array<any>): any;
	getMe(): Promise<any>;
	getUserProfilePhotos(uid: any): Promise<any>;
	getFile(file_id: string): Promise<any>;
	sendMessage(chat_id: string | number, text: string, parameters?: any): Promise<any>;
	editMessageText(chat_id: string | number, message_id: number, text: string, parameters?: any): Promise<any>;
	editMessageReplyMarkup(chat_id: string | number, message_id: number, parameters?: any): Promise<any>;
	forwardMessage(chat_id: string | number, from_chat_id: string | number, message_id: number): Promise<any>;
	answerCallbackQuery(callback_query_id: string, text?: string, show_alert?: boolean, url?: string): Promise<any>;
	setWebhook(url: string, certificate?: string, max_connections?: number, allowed_updates?: Array<string>): Promise<any>;
	getWebhookInfo(): Promise<ResponseFromServer<getWebhookInfoResult>>;
	sendDocument(chat_id: string | number, document: InMemoryDocument, caption?: string): Promise<any>;
	sendChatAction(chat_id: string | number, action: 'typing' | 'record_video_note' | 'upload_video_note' | 'find_location' | 'upload_document' | 'upload_audio' | 'record_audio' | 'upload_video' | 'record_video' | 'upload_photo'): Promise<any>;
	sendInvoice(chat_id: string, provider_token: string, invoice: any, start_parameter: string, options?: any): Promise<any>;
	answerPreCheckoutQuery(pre_checkout_query_id: string, message?: string): Promise<any>;
	private post;
}
export declare class TelegramBot extends Resolvable<BotSettings> {
	protected telegram: TelegramClient;
	setupWebhook(url: string): Promise<any>;
	getMe(): Promise<any>;
	sendText(chat_id: any, text: string, parameters?: any): Promise<any>;
	sendCode(chat_id: any, text: string, parameters?: any): Promise<any>;
}
export declare class TelegramBotContext<S extends BotSettings> extends BotContext<S> {
	readonly telegram: TelegramBot;
	readonly identity: TelegramMessageIdentity;
	readonly event: TelegramBotEvent;
	readonly message: TelegramBotMessage;
	sendTextWithKeyboard(): void;
	sendTextWithInlineLayout(): void;
}
declare class FileTokenModel extends Model {
	file: string;
	type: string;
	byteLength: number;
	title: string;
	originalname: string;
	originalurl: string;
}
declare class FileModel extends Model implements IBlobLike {
	type: string;
	byteLength: number;
	data: Buffer;
	width: number;
	height: number;
	duration: number;
	ipfs: string;
}
export interface WebFileMetadata {
	type?: string;
	title?: string;
	originalname?: string;
	originalurl?: string;
	width?: number;
	height?: number;
	duration?: number;
	ipfs?: string;
}
export interface WebFileReference extends WebFileMetadata {
	id: string;
	type: string;
	byteLength: number;
	url: string;
}
export declare class WebFileService {
	findTokenById(ctx: Ctx, id: string): Promise<FileTokenModel | undefined>;
	findFileById(ctx: Ctx, id: string): Promise<FileModel | undefined>;
	updateOne(ctx: Ctx, id: string, metadata: WebFileMetadata): Promise<boolean>;
	insertFromURL(ctx: Ctx, url: string, metadata?: WebFileMetadata): Promise<WebFileReference>;
	insertFromBuffer(ctx: Ctx, data: Buffer, metadata: WebFileMetadata): Promise<WebFileReference>;
}
export declare class WebFileController {
	protected WebFile: WebFileService;
	token(ctx: WebCtx): Promise<void>;
}